/*
Copyright © 2014 Andreas Enge <andreas.enge@inria.fr>

This file is part of pari-gnump.

Pari-gnump is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at
your option) any later version.

Pari-gnump is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Pari-gnump.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "pari-gnump-user.h"

/* The following functions perform floating point computations using the
   mpfr and mpc libraries. They take as additional parameter a long prec,
   coding a pari floating point precision which is used for the result.
   Floating point arguments are first transformed into variables of mpfr
   type, using their own precision; the parameter prec is also used to
   determine the floating point precision of exact arguments of type t_INT
   or t_FRAC, for which there is no intrinsic way of deriving a precision.

   If such a function is used inside GP, it should be installed with a
   parameter code of "p" for the prec parameter, which is then automagically
   replaced by the current default precision.
*/

/****************************************************************************/

GEN pari_mpc_mul (GEN x, GEN y, long prec)
   /* Returns the product of x and y.
      Install into GP with
      install ("pari_mpc_mul", "GGp", "mpc_mul", "./libpari-gnump.so");
   */

{
   mpfr_prec_t p = bit_accuracy (prec);
   mpc_t z, z1, z2;

   pari_mpc_init2 (z, p);
   pari_mpc_init_set_GEN (z1, x, p);
   pari_mpc_init_set_GEN (z2, y, p);

   mpc_mul (z, z1, z2, MPC_RNDNN);

   return mpc_get_GEN (z);
}

/****************************************************************************/

GEN pari_mpfr_mul (GEN x, GEN y, long prec)
   /* Returns the product of x and y.
      Install into GP with
      install ("pari_mpfr_mul", "GGp", "mpfr_mul", "./libpari-gnump.so");
   */

{
   mpfr_prec_t p = bit_accuracy (prec);
   mpfr_t z, z1, z2;
   
   pari_mpfr_init2 (z, p);
   pari_mpfr_init_set_GEN (z1, x, p);
   pari_mpfr_init_set_GEN (z2, y, p);

   mpfr_mul (z, z1, z2, MPFR_RNDN);
   
   return mpfr_get_GEN (z);
}

/****************************************************************************/

GEN pari_mpfr_erf (GEN x, long prec)
   /* Returns the error function at x.
      Install into GP with
      install ("pari_mpfr_erf", "Gp", "mpfr_erf", "./libpari-gnump.so");
   */

{
   mpfr_prec_t p = bit_accuracy (prec);
   mpfr_t z, z1;

   pari_mpfr_init2 (z, p);
   pari_mpfr_init_set_GEN (z1, x, p);

   mpfr_erf (z, z1, MPFR_RNDN);

   return mpfr_get_GEN (z);
}

/****************************************************************************/

GEN pari_mpfr_zeta (GEN x, long prec)
   /* Returns the Riemann zeta function at x.
      Install into GP with
      install ("pari_mpfr_zeta", "Gp", "mpfr_zeta", "./libpari-gnump.so");
   */

{
   mpfr_prec_t p = bit_accuracy (prec);
   mpfr_t z, z1;

   pari_mpfr_init2 (z, p);
   pari_mpfr_init_set_GEN (z1, x, p);

   mpfr_zeta (z, z1, MPFR_RNDN);

   return mpfr_get_GEN (z);
}

/****************************************************************************/
